/*
 * NVIDIA_COPYRIGHT_BEGIN
 *
 * Copyright 1999-2002 by NVIDIA Corporation.  All rights reserved.  All
 * information contained herein is proprietary and confidential to NVIDIA
 * Corporation.  Any use, reproduction, or disclosure without the written
 * permission of NVIDIA Corporation is prohibited.
 *
 * NVIDIA_COPYRIGHT_END
 */

/**
 * @file glr_comm.h
 * @brief Common C++ and JavaScript declarations for GL Random tests.
 */

#if !defined(__cplusplus)
var glr_commId = "$Id$";
#endif

/** <a name="Dimensions"> </a> */
/*
 * Dimensions of our world, before the view projection into eye-space.
 *
 * We use a large world, so that we get sub-pixel resolution even when
 * using 16-bit integer vertex coordinates.
 * Currently, we keep model-space the same as world-space.
 *
 * The center of the world is:   0,0,0
 *
 * The eye is at world location: 0,0,16000
 */
#define RND_GEOM_WorldWidth   (20 * 1024)
#define RND_GEOM_WorldHeight  (15 * 1024)
#define RND_GEOM_WorldDepth   (15 * 1024)
#define RND_GEOM_WorldLeft    (-RND_GEOM_WorldWidth/2)
#define RND_GEOM_WorldRight   ( RND_GEOM_WorldWidth/2)
#define RND_GEOM_WorldBottom  (-RND_GEOM_WorldHeight/2)
#define RND_GEOM_WorldTop     ( RND_GEOM_WorldHeight/2)
#define RND_GEOM_WorldFar     (3*RND_GEOM_WorldDepth/2)  // absolute eye distance for glOrtho() or fog
#define RND_GEOM_WorldNear    (RND_GEOM_WorldDepth/2)    // absolute eye distance for glOrtho() or fog

/** <a name="Modules"> </a> */
/*
 * Codes for the different randomizer modules:
 */
#define RND_CMB              0   // module RndCombiner,       r_combin.cpp
#define RND_EVAL             1   // (deleted)
#define RND_FOG              2   // module RndFog,            r_fog.cpp
#define RND_FRAG             3   // module RndFragment,       r_frag.cpp
#define RND_GEOM             4   // module RndGeometry,       r_geom.cpp
#define RND_LIGHT            5   // module RndLighting,       r_light.cpp
#define RND_PRIM             6   // module RndPrimitiveModes, r_primmo.cpp
#define RND_TX               7   // module RndTexturing,      r_tex.cpp
#define RND_VX               8   // module RndVertexes,       r_vx.cpp
#define RND_VXFMT            9   // module RndVertexFormat,   r_vxfmt.cpp
#define RND_TSTCTRL         10   // module GLRandom,          glrandom.cpp
#define RND_NUM_RANDOMIZERS 11
#define RND_NO_RANDOMIZER   -1

/** <a name="Indexes"> </a> */
/*
 * Picker Indexes  (JavaScript)
 *
 *
 * These are the legal picker indexes for GLRandom.SetPicker(), .GetPicker(),
 * and .SetDefaultPicker().
 * Use them to control the way GLRandom does its random picking.
 *
 * EXAMPLES:
 *
 *    To always use both texture units 0 and 3:
 *
 *       GLRandom.SetPicker(RND_TXU_ENABLE_MASK, [  0x9 ]);
 *
 *    To enable fog 75% of the time:
 *
 *       GLRandom.SetPicker(RND_FOG_ENABLE, [ [3, GL_TRUE], [1, GL_FALSE] ]);
 *
 *    To force vertex primary/material color to r,g,b 0,0,255:
 *
 *       GLRandom.SetPicker(RND_VX_COLOR_BYTE, [ "list", 0,0,255 ]);
 *
 *    To allow any light specular exponent (shininess) between 0 and 256,
 *    but make higher numbers more probable than lower numbers:
 *
 *       GLRandom.SetPicker(RND_LIGHT_SHININESS, [ [3, 0,128], [97, 129,256] ]);
 *
 *
 * FancyPicker configuration syntax:  (JavaScript)
 *
 *    [ MODE, OPTIONS, ITEMS ]
 *
 * MODE must be a quoted string, from this list:
 *
 *    "const" "constant"         (always returns same value)
 *    "rand"  "random"           (return a random value)
 *    "list"                     (return next item in a list)
 *    "step"                     (schmoo mode, i.e. begin/step/end )
 *    "js"                       (call a JavaScript function, flexible but slow)
 *
 * OPTIONS must be zero or more quoted strings, from this list:
 *
 *    "PickOnRestart"            (pick on Restart loops, re-use that value for
 *                                all other loops.  Otherwise, we pick a new
 *                                value on each loop.)
 *    "clamp" "wrap"             (for LIST or STEP mode, what to do at the end
 *                                of the sequence: stay at end, or go back to
 *                                the beginning)
 *    "int" "float"              (whether to interpret the numbers as floats or
 *                                as integers.  If you don't specify, uses the
 *                                default value from C++, which is usually best)
 *
 *    1) Capitalization is ignored.
 *    2) If you specify more than one mode, the last one "wins".
 *    3) If you fail to specify any mode at all, the program tries to guess from
 *       the format of ITEMS.
 *    4) You are actually allowed to specify MODE and OPTIONS strings
 *       in any order, as long as they all preceed the ITEMS.
 *
 * ITEMS depends on what mode you are in:
 *
 *    CONST ITEM     must be a single number.
 *
 *    RANDOM ITEMS   must be a sequence of one or more arrays, each like this:
 *                      [ relative_probability, value ]     or
 *                      [ relative_probability, min, max ]
 *
 *    LIST ITEMS     must be a sequence of one or more numbers or arrays:
 *                      value  or  [ value ]  or  [ repeat_count, value ]
 *
 *    STEP ITEMS     must be three numbers:  begin, step, end
 *
 *    JS ITEMS       must be one or two javascript function names:
 *                      pick_func  or  pick_func, restart_func
 *
 * Picker Indexes  (C++)
 *
 *
 * We keep a separate array of Utility::FancyPicker objects in each
 * randomizer module.
 *
 * In C++, strip away the base offset, i.e. (index % RND_BASE_OFFSET).
 *
 * We set default values from C++, but the JavaScript settings override them.
 *
 * WARNING: keep the RND_*_NUM_PICKERS defines up-to-date, or you will
 *          corrupt memory at runtime (asserts in the debug build).
 */

#define RND_BASE_OFFSET    100
#if defined(__cplusplus)
   // C++
   #define IDX(i)  (i % RND_BASE_OFFSET)
#else
   // JavaScript
   #define IDX(i)  i
#endif

#define RND_CMB_THEME                     IDX(  0)   // which canned register-combiner program to use
#define RND_CMB_THEME_Modulate                  0    //  value for RND_CMB_THEME
#define RND_CMB_THEME_NormalMap                 1    //  value for RND_CMB_THEME
#define RND_CMB_THEME_Decal                     2    //  value for RND_CMB_THEME
#define RND_CMB_THEME_Disabled                  3    //  value for RND_CMB_THEME
#define RND_CMB_THEME_4WayBlend                 4    //  value for RND_CMB_THEME
#define RND_CMB_THEME_Random                    5    //  value for RND_CMB_THEME
#define RND_CMB_THEME_IgnoreTextures            6    //  value for RND_CMB_THEME
#define RND_CMB_COLOR                     IDX(  1)   // color 0-255
#define RND_CMB_ALPHA                     IDX(  2)   // transparency, 0-255
#define RND_CMB_NUM_STAGES                IDX(  3)   // how many general combiner stages to enable
#define RND_CMB_OUT_SCALE_BIAS            IDX(  4)   // general combiner stage output scale/bias
#define RND_CMB_OUT_SCALE_BIAS_HalfX            0    // output = 0.5 * output         (all are clamped to -1,1)
#define RND_CMB_OUT_SCALE_BIAS_X                1    // output = output
#define RND_CMB_OUT_SCALE_BIAS_XMinusHalf       2    // output = output - 0.5
#define RND_CMB_OUT_SCALE_BIAS_2X               3    // output = 2 * output
#define RND_CMB_OUT_SCALE_BIAS_2XMinusHalf      4    // output = 2 * (output - 0.5)
#define RND_CMB_OUT_SCALE_BIAS_4X               5    // output = 4 * output
#define RND_CMB_OP                        IDX(  5)   // general combiner stage operation
#define RND_CMB_OP_MultMultSum                  0    // outputs are AB,    CD,    AB+CD
#define RND_CMB_OP_MultMultMux                  1    // outputs are AB,    CD,    AB mux CD
#define RND_CMB_OP_DotMult                      2    // outputs are AdotB, CD,    (discard)
#define RND_CMB_OP_MultDot                      3    // outputs are AB,    CdotD, (discard)
#define RND_CMB_OP_DotDot                       4    // outputs are AdotB, CdotD, (discard)
#define RND_CMB_OP_MultMultSumSGE               5    // outputs are AB,    CD,    AB+CD (force to 0, or 1 if >=0)
#define RND_CMB_OP_MultMultMuxSGE               6    // outputs are AB,    CD,    AB mux CD (force to 0, or 1 if >= 0)
#define RND_CMB_INPUT_REG                 IDX(  6)   // which input register for a general combiner
#define RND_CMB_INPUT_COMPONENT           IDX(  7)   // alpha or rgb for input to general combiner stage
#define RND_CMB_INPUT_MAPPING             IDX(  8)   // input mapping for general combiner stage
#define RND_CMB_OUTPUT_REG                IDX(  9)   // which output register for a general combiner
#define RND_CMB_OP_A                      IDX( 10)   // general combiner stage operation (alpha)
#define RND_CMB_FINAL_REG                 IDX( 11)   // input register to find RGB combiner
#define RND_CMB_FINAL_MAPPING             IDX( 12)   // input mapping for final combiner stage
#define RND_CMB_CLAMP_COLOR_SUM           IDX( 13)   // do/dont clamp the color sum in final combiner stage
#define RND_CMB_FORCE_FINAL               IDX( 14)   // do/don't force a "vanilla" final combiner program
#define RND_CMB_NUM_PICKERS                    15

#define RND_EVAL_NUM_PICKERS                    0

#define RND_FOG_ENABLE                    IDX(200)   // enable/disable depth fog
#define RND_FOG_DIST_MODE                 IDX(201)   // fog distance mode (eye-plane, eye-plane-abs, eye-radial)
#define RND_FOG_MODE                      IDX(202)   // fog intensity calculation mode: linear/exp/exp2
#define RND_FOG_DENSITY                   IDX(203)   // coefficient for exp/exp2 modes
#define RND_FOG_START                     IDX(204)   // starting eye-space Z depth for linear mode
#define RND_FOG_END                       IDX(205)   // ending eye-space Z depth for linear mode
#define RND_FOG_COORD                     IDX(206)   // where does fog distance come from? explicit per-vertex fog coord or fragment Z?
#define RND_FOG_COLOR                     IDX(207)   // Fog color (float, called 3 times per loop)
#define RND_FOG_NUM_PICKERS                     8

#define RND_FRAG_DEPTH_TEST               IDX(300)   // do/don't enable Z buffer hidden-surface removal
#define RND_FRAG_ALPHA_FUNC               IDX(301)   // fragment alpha test function
#define RND_FRAG_BLEND_SOURCE_FUNC        IDX(302)   // scaling function for source (fragment) color in blending
#define RND_FRAG_BLEND_EQN                IDX(303)   // what op to do with the scaled source and dest (normally addition)
#define RND_FRAG_DEPTH_FUNC               IDX(304)   // fragment Z test function
#define RND_FRAG_LOGIC_OP                 IDX(305)   // fragment logic op
#define RND_FRAG_STENCIL_FUNC             IDX(306)   // comparison with stencil buffer for fragment testing
#define RND_FRAG_STENCIL_OP               IDX(307)   // what to do to the stencil buffer when op passes/fail
#define RND_FRAG_SCISSOR_ENABLE           IDX(308)   // enable scissor box that prevents drawing to the outer 5 pixels of the screen.
#define RND_FRAG_BLEND_COLOR              IDX(309)   // blend color r,g, or b (float)
#define RND_FRAG_BLEND_ALPHA              IDX(310)   // blend color alpha value (float)
#define RND_FRAG_STENCIL_2SIDE            IDX(311)   // use the NV_stencil_two_side extension (if present)
#define RND_FRAG_DEPTH_CLAMP              IDX(312)   // use the NV_depth_clamp extension to clamp at the near/far planes instead of clipping.
#define RND_FRAG_VEIWPORT_OFFSET_X        IDX(313)   // Offset and reduce the viewport by this many pixels in x
#define RND_FRAG_VEIWPORT_OFFSET_Y        IDX(314)   // Offset and reduce the viewport by this many pixels in y
#define RND_FRAG_DEPTH_BOUNDS_TEST        IDX(315)   // do/don't enable GL_NV_depth_bounds_test
#define RND_FRAG_NUM_PICKERS                   16

#define RND_GEOM_BBOX_WIDTH               IDX(400)   // X width of bounding-box for each primitive
#define RND_GEOM_BBOX_HEIGHT              IDX(401)   // Y height of bounding-box for each primitive
#define RND_GEOM_BBOX_XOFFSET             IDX(402)   // X offset of bbox from previous bbox within row
#define RND_GEOM_BBOX_YOFFSET             IDX(403)   // Y offset of new bbox row from previous bbox row
#define RND_GEOM_BBOX_Z_BASE              IDX(404)   // Z center of BBOX (offset from max depth)
#define RND_GEOM_BBOX_DEPTH               IDX(405)   // Z depth of bounding box for each primitive
#define RND_GEOM_FRONT_FACE               IDX(406)   // is a clockwise poly front, or CCWise?
#define RND_GEOM_VTX_NORMAL_SCALING       IDX(407)   // how should vertex normals be forced to length 1.0?
#define RND_GEOM_VTX_NORMAL_SCALING_Unscaled    0    // GL will use my vertex normals as-is
#define RND_GEOM_VTX_NORMAL_SCALING_Scaled      1    // GL will scale my normals based on my modelview matrix
#define RND_GEOM_VTX_NORMAL_SCALING_Corrected   2    // GL will correct my normals to length 1.0
#define RND_GEOM_CULL_FACE                IDX(408)   // what faces to cull? (none, front, back, both)
#define RND_GEOM_BBOX_FACES_FRONT         IDX(409)   // should the polygons drawn in this bounding box face front (else back).
#define RND_GEOM_FORCE_BIG_SIMPLE         IDX(410)   // Force polygons to be big and simple (for simplifying debugging)
#define RND_GEOM_TXU_GEN_ENABLED          IDX(411)   // should texture coordinate generation be enabled for this texture unit?
#define RND_GEOM_TXU_GEN_MODE             IDX(412)   // what texture coord. gen. mode?
#define RND_GEOM_TXMX_OPERATION           IDX(413)   // what should we do with the texture matrix?
#define RND_GEOM_TXMX_OPERATION_Identity        0    //   value for RND_GEOM_TXMX_OPERATION
#define RND_GEOM_TXMX_OPERATION_2dRotate        1    //   value for RND_GEOM_TXMX_OPERATION
#define RND_GEOM_TXMX_OPERATION_2dTranslate     2    //   value for RND_GEOM_TXMX_OPERATION
#define RND_GEOM_TXMX_OPERATION_2dScale         3    //   value for RND_GEOM_TXMX_OPERATION
#define RND_GEOM_TXMX_OPERATION_2dAll           4    //   value for RND_GEOM_TXMX_OPERATION
#define RND_GEOM_TXMX_OPERATION_Project         5    //   value for RND_GEOM_TXMX_OPERATION
#define RND_GEOM_VERTEX_WEIGHTING_ENABLE  IDX(414)   // enable the nv10/11/15 vertex weighting/skinning feature
#define RND_GEOM_USE_PERSPECTIVE          IDX(415)   // Use perspective projection (else orthographic).
#define RND_GEOM_VXPGM_ENABLE             IDX(416)   // enable vertex program
#define RND_GEOM_VXPGM_INDEX              IDX(417)   // which vertex program to use
#define RND_GEOM_VXPGM_INDEX_UseAttrWGHT        0    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseAttrNRML        1    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseAttrCOL0        2    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseAttrFOGC        3    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseAttr6           4    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseAttr7           5    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseAttrTEX4        6    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseAttrTEX5        7    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseAttrTEX6        8    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseAttrTEX7        9    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_IdxLight012       10    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_IdxLight345       11    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_IdxLight678       12    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_IdxLight91011     13    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseADD            14    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseRCP            15    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseDST            16    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseMIN            17    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseMAX            18    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseSLT            19    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseSGE            20    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseEXP            21    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseLOG            22    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseDPH            23    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_UseRCC            24    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_MaxLength         25    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_PosInvar          26    //   value for RND_GEOM_VXPGM_INDEX
#define RND_GEOM_VXPGM_INDEX_NumFixedPgms      27
#define RND_GEOM_VXPGM_INDEX_User            1000    //   Use one of the user-specified programs.
#define RND_GEOM_VXPGM_INDEX_Random          2000    //   Use one of the randomly-generated programs.
#define RND_GEOM_VXPGM_LIGHT0             IDX(418)   // which "light" to use in certain vertex programs
#define RND_GEOM_VXPGM_LIGHT1             IDX(419)   // which "light" to use in certain vertex programs
#define RND_GEOM_VXPGM_POINTSZ            IDX(420)   // do/don't generate point size from vertex program
#define RND_GEOM_VXPGM_TWOSIDE            IDX(421)   // do/don't calculate two-sided lighting in vertex programs
#define RND_GEOM_USE_VX_STATE_PGM         IDX(422)   // do/don't use a vertex state program to shuffle vx prog constants around
#define RND_GEOM_VXPGM_POINTSZ_VAL        IDX(423)   // (float) point size used inside vx progs
#define RND_GEOM_USER_CLIP_PLANE_MASK     IDX(424)   // which user clip planes to enable? (bit mask)
#define RND_GEOM_VXPGM_NUMRANDOM          IDX(425)   // how many random vertex programs to generate per frame
#define RND_GEOM_VXPGM_NUMRANDOM_Max          100    //   max value allowed
#define RND_GEOM_VXPGM_NUM_OPS            IDX(426)   // how many instructions long should a random vxpgm be?
#define RND_GEOM_VXPGM_COL0_ALIAS         IDX(427)   // which vertex attrib will contain input primary color?
#define RND_GEOM_VXPGM_POSINVAR           IDX(428)   // use OPTION NV_position_invariant (vp1.1 and up)
#define RND_GEOM_VXPGM_OP                 IDX(429)   // random vertex program instruction
#define RND_GEOM_VXPGM_OP_ABS                   0    // ABS[C]         v       v        absolute value
#define RND_GEOM_VXPGM_OP_ADD                   1    // ADD[C]         v,v     v        add
#define RND_GEOM_VXPGM_OP_ARA                   2    // ARA[C]         av      av       address register add
#define RND_GEOM_VXPGM_OP_ARL                   3    // ARL[C]         v       av       address register load
#define RND_GEOM_VXPGM_OP_ARR                   4    // ARR[C]         v       av       address register load (with round)
#define RND_GEOM_VXPGM_OP_BRA                   5    // BRA            as      none     branch
#define RND_GEOM_VXPGM_OP_CAL                   6    // CAL            as      none     subroutine call
#define RND_GEOM_VXPGM_OP_COS                   7    // COS[C]         s       ssss     cosine
#define RND_GEOM_VXPGM_OP_DP3                   8    // DP3[C]         v,v     ssss     3-component dot product
#define RND_GEOM_VXPGM_OP_DP4                   9    // DP4[C]         v,v     ssss     4-component dot product
#define RND_GEOM_VXPGM_OP_DPH                  10    // DPH[C]         v,v     ssss     homogeneous dot product
#define RND_GEOM_VXPGM_OP_DST                  11    // DST[C]         v,v     v        distance vector
#define RND_GEOM_VXPGM_OP_EX2                  12    // EX2[C]         s       ssss     exponential base 2
#define RND_GEOM_VXPGM_OP_EXP                  13    // EXP[C]         s       v        exponential base 2 (approximate)
#define RND_GEOM_VXPGM_OP_FLR                  14    // FLR[C]         v       v        floor
#define RND_GEOM_VXPGM_OP_FRC                  15    // FRC[C]         v       v        fraction
#define RND_GEOM_VXPGM_OP_LG2                  16    // LG2[C]         s       ssss     logarithm base 2
#define RND_GEOM_VXPGM_OP_LIT                  17    // LIT[C]         v       v        compute light coefficients
#define RND_GEOM_VXPGM_OP_LOG                  18    // LOG[C]         s       v        logarithm base 2 (approximate)
#define RND_GEOM_VXPGM_OP_MAD                  19    // MAD[C]         v,v,v   v        multiply and add
#define RND_GEOM_VXPGM_OP_MAX                  20    // MAX[C]         v,v     v        maximum
#define RND_GEOM_VXPGM_OP_MIN                  21    // MIN[C]         v,v     v        minimum
#define RND_GEOM_VXPGM_OP_MOV                  22    // MOV[C]         v       v        move
#define RND_GEOM_VXPGM_OP_MUL                  23    // MUL[C]         v,v     v        multiply
#define RND_GEOM_VXPGM_OP_RCC                  24    // RCC[C]         s       ssss     reciprocal (clamped)
#define RND_GEOM_VXPGM_OP_RCP                  25    // RCP[C]         s       ssss     reciprocal
#define RND_GEOM_VXPGM_OP_RET                  26    // RET            none    none     subroutine call return
#define RND_GEOM_VXPGM_OP_RSQ                  27    // RSQ[C]         s       ssss     reciprocal square root
#define RND_GEOM_VXPGM_OP_SEQ                  28    // SEQ[C]         v,v     v        set on equal
#define RND_GEOM_VXPGM_OP_SFL                  29    // SFL[C]         v,v     v        set on false
#define RND_GEOM_VXPGM_OP_SGE                  30    // SGE[C]         v,v     v        set on greater than or equal
#define RND_GEOM_VXPGM_OP_SGT                  31    // SGT[C]         v,v     v        set on greater than
#define RND_GEOM_VXPGM_OP_SIN                  32    // SIN[C]         s       ssss     sine
#define RND_GEOM_VXPGM_OP_SLE                  33    // SLE[C]         v,v     v        set on less than or equal
#define RND_GEOM_VXPGM_OP_SLT                  34    // SLT[C]         v,v     v        set on less than
#define RND_GEOM_VXPGM_OP_SNE                  35    // SNE[C]         v,v     v        set on not equal
#define RND_GEOM_VXPGM_OP_SSG                  36    // SSG[C]         v       v        set sign
#define RND_GEOM_VXPGM_OP_STR                  37    // STR[C]         v,v     v        set on true
#define RND_GEOM_VXPGM_OP_SUB                  38    // SUB[C]         v,v     v        subtract
#define RND_GEOM_VXPGM_OP_NUM_OPCODES          39
#define RND_GEOM_VXPGM_OUT_CC             IDX(430)   // Output register condition code.
#define RND_GEOM_VXPGM_OUT_CC_eq                0    //   value for RND_TX_FRAG_PROG_OUT_CC
#define RND_GEOM_VXPGM_OUT_CC_ge                1    //   value for RND_TX_FRAG_PROG_OUT_CC
#define RND_GEOM_VXPGM_OUT_CC_gt                2    //   value for RND_TX_FRAG_PROG_OUT_CC
#define RND_GEOM_VXPGM_OUT_CC_le                3    //   value for RND_TX_FRAG_PROG_OUT_CC
#define RND_GEOM_VXPGM_OUT_CC_lt                4    //   value for RND_TX_FRAG_PROG_OUT_CC
#define RND_GEOM_VXPGM_OUT_CC_ne                5    //   value for RND_TX_FRAG_PROG_OUT_CC
#define RND_GEOM_VXPGM_OUT_CC_tr                6    //   value for RND_TX_FRAG_PROG_OUT_CC
#define RND_GEOM_VXPGM_OUT_CC_fl                7    //   value for RND_TX_FRAG_PROG_OUT_CC
#define RND_GEOM_VXPGM_OUT_CC_none              8    //   value for RND_TX_FRAG_PROG_OUT_CC
#define RND_GEOM_VXPGM_OUT_CC_NUM_COND          8
#define RND_GEOM_VXPGM_UPDATE_CC          IDX(431)   // do/don't update the CC (condition code) in an instruction in a random vertex program
#define RND_GEOM_VXPGM_SWIZZLE            IDX(432)   // input or CC swizzle
#define RND_GEOM_VXPGM_SWIZZLE_xyzw             1    //   value for RND_TX_FRAG_PROG_SWIZZLE
#define RND_GEOM_VXPGM_SWIZZLE_shuffle          2    //   value for RND_TX_FRAG_PROG_SWIZZLE
#define RND_GEOM_VXPGM_SWIZZLE_random           3    //   value for RND_TX_FRAG_PROG_SWIZZLE
#define RND_GEOM_VXPGM_OUT_REG            IDX(433)   // output register for random vertex program instruction
#define RND_GEOM_VXPGM_REG_vOPOS                0    // object-space position
#define RND_GEOM_VXPGM_REG_vWGHT                1    // skinning weight
#define RND_GEOM_VXPGM_REG_vNRML                2    // object-space normal
#define RND_GEOM_VXPGM_REG_vCOL0                3    // primary color
#define RND_GEOM_VXPGM_REG_vCOL1                4    // secondary color
#define RND_GEOM_VXPGM_REG_vFOGC                5    // fog coord
#define RND_GEOM_VXPGM_REG_v6                   6    // extra
#define RND_GEOM_VXPGM_REG_v7                   7    // extra
#define RND_GEOM_VXPGM_REG_vTEX0                8    // texture coord
#define RND_GEOM_VXPGM_REG_vTEX1                9    //
#define RND_GEOM_VXPGM_REG_vTEX2               10    //
#define RND_GEOM_VXPGM_REG_vTEX3               11    //
#define RND_GEOM_VXPGM_REG_vTEX4               12    //
#define RND_GEOM_VXPGM_REG_vTEX5               13    //
#define RND_GEOM_VXPGM_REG_vTEX6               14    //
#define RND_GEOM_VXPGM_REG_vTEX7               15    //
#define RND_GEOM_VXPGM_REG_oHPOS               16    // Homogeneous clip space position    (x,y,z,w)
#define RND_GEOM_VXPGM_REG_oCOL0               17    // Primary color (front-facing)       (r,g,b,a)
#define RND_GEOM_VXPGM_REG_oCOL1               18    // Secondary color (front-facing)     (r,g,b,a)
#define RND_GEOM_VXPGM_REG_oBFC0               19    // Back-facing primary color          (r,g,b,a)
#define RND_GEOM_VXPGM_REG_oBFC1               20    // Back-facing secondary color        (r,g,b,a)
#define RND_GEOM_VXPGM_REG_oFOGC               21    // Fog coordinate                     (f,*,*,*)
#define RND_GEOM_VXPGM_REG_oPSIZ               22    // Point size                         (p,*,*,*)
#define RND_GEOM_VXPGM_REG_oTEX0               23    // Texture coordinate set 0           (s,t,r,q)
#define RND_GEOM_VXPGM_REG_oTEX1               24    // Texture coordinate set 1           (s,t,r,q)
#define RND_GEOM_VXPGM_REG_oTEX2               25    // Texture coordinate set 2           (s,t,r,q)
#define RND_GEOM_VXPGM_REG_oTEX3               26    // Texture coordinate set 3           (s,t,r,q)
#define RND_GEOM_VXPGM_REG_oTEX4               27    // Texture coordinate set 4           (s,t,r,q)
#define RND_GEOM_VXPGM_REG_oTEX5               28    // Texture coordinate set 5           (s,t,r,q)
#define RND_GEOM_VXPGM_REG_oTEX6               29    // Texture coordinate set 6           (s,t,r,q)
#define RND_GEOM_VXPGM_REG_oTEX7               30    // Texture coordinate set 7           (s,t,r,q)
#define RND_GEOM_VXPGM_REG_oCLP0               31    // clip coord (vp2 only)
#define RND_GEOM_VXPGM_REG_oCLP1               32    //
#define RND_GEOM_VXPGM_REG_oCLP2               33    //
#define RND_GEOM_VXPGM_REG_oCLP3               34    //
#define RND_GEOM_VXPGM_REG_oCLP4               35    //
#define RND_GEOM_VXPGM_REG_oCLP5               36    //
#define RND_GEOM_VXPGM_REG_R0                  37    // temp register
#define RND_GEOM_VXPGM_REG_R1                  38    //
#define RND_GEOM_VXPGM_REG_R2                  39    //
#define RND_GEOM_VXPGM_REG_R3                  40    //
#define RND_GEOM_VXPGM_REG_R4                  41    //
#define RND_GEOM_VXPGM_REG_R5                  42    //
#define RND_GEOM_VXPGM_REG_R6                  43    //
#define RND_GEOM_VXPGM_REG_R7                  44    //
#define RND_GEOM_VXPGM_REG_R8                  45    //
#define RND_GEOM_VXPGM_REG_R9                  46    //
#define RND_GEOM_VXPGM_REG_R10                 47    //
#define RND_GEOM_VXPGM_REG_R11                 48    //
#define RND_GEOM_VXPGM_REG_R12                 49    // temp register (vp2 only)
#define RND_GEOM_VXPGM_REG_R13                 50    //
#define RND_GEOM_VXPGM_REG_R14                 51    //
#define RND_GEOM_VXPGM_REG_R15                 52    //
#define RND_GEOM_VXPGM_REG_CC                  53    // bit bucket register (vp2 only)
#define RND_GEOM_VXPGM_REG_Const               54    // place holder for the 96 (or 256) const regs
#define RND_GEOM_VXPGM_REG_A0                  55    // Address 0 register (special)
#define RND_GEOM_VXPGM_REG_A1                  56    // Address 1 register (special, vp2 only)
#define RND_GEOM_VXPGM_REG_NUM_REGS            57
#define RND_GEOM_VXPGM_OUT_MASK           IDX(434)   // output register component mask
#define RND_GEOM_VXPGM_CONST_RELADDR      IDX(435)   // do/don't use relative addressing for const reg reads
#define RND_GEOM_VXPGM_IN_ABS             IDX(436)   // do/don't take abs. value of input argument
#define RND_GEOM_VXPGM_IN_NEG             IDX(437)   // do/don't negate input argument
#define RND_GEOM_MODELVIEW_OPERATION      IDX(438)   // what should we do with the modelview matrix?
#define RND_GEOM_MODELVIEW_OPERATION_Identity   0
#define RND_GEOM_MODELVIEW_OPERATION_Translate  1
#define RND_GEOM_MODELVIEW_OPERATION_Scale      2
#define RND_GEOM_MODELVIEW_OPERATION_Rotate     3
#define RND_GEOM_NUM_PICKERS                   39

#define RND_LIGHT_ENABLE                  IDX(500)   // enable/not lighting
#define RND_LIGHT_SHADE_MODEL             IDX(501)   // flat or smooth
#define RND_LIGHT_TYPE                    IDX(502)   // directional, point-source, spotlight
#define RND_LIGHT_TYPE_Directional              0    // light with no local position, just a direction (like sunlight)
#define RND_LIGHT_TYPE_PointSource              1    // light with local position, radiating in all directions equally
#define RND_LIGHT_TYPE_SpotLight                2    // local positioned light, radiating in a cone
#define RND_LIGHT_SPOT_CUTOFF             IDX(503)   // angle (in degrees) at which spotlight cuts off
#define RND_LIGHT_NUM_ON                  IDX(504)   // how many light sources should be enabled?
#define RND_LIGHT_BACK                    IDX(505)   // does this light shine on the front or back of polygons?
#define RND_LIGHT_MODEL_LOCAL_VIEWER      IDX(506)   // should we calculate specular highlights with a custom eye angle for each vertex?
#define RND_LIGHT_MODEL_TWO_SIDE          IDX(507)   // should we light only front, or both front and back sides of polygons?
#define RND_LIGHT_MODEL_COLOR_CONTROL     IDX(508)   // should specular highlight color be calculated independently of texturing?
#define RND_LIGHT_MAT_IS_EMISSIVE         IDX(509)   // should this material "glow"?
#define RND_LIGHT_MAT                     IDX(510)   // which material color property should glColor() control?
#define RND_LIGHT_SHININESS               IDX(511)   // shininess (specular exponent) for lighting
#define RND_LIGHT_COLOR                   IDX(512)   // light color (specular & diffuse) float, 3 calls per light
#define RND_LIGHT_AMBIENT                 IDX(513)   // light color (ambient) float, 3 calls per light
#define RND_LIGHT_ALPHA                   IDX(514)   // light alpha value (float)
#define RND_LIGHT_SPOT_EXPONENT           IDX(515)   // spotlight falloff exponent
#define RND_LIGHT_NUM_PICKERS                  16

#define RND_PRIM_POINT_SMOOTH_ENABLE      IDX(600)   // antialiasing enable for points
#define RND_PRIM_POINT_SIZE               IDX(601)   // point size (float)
#define RND_PRIM_POINT_ATTEN_ENABLE       IDX(602)   // enable the point parameters extension for distance attenuation of points
#define RND_PRIM_POINT_SIZE_MIN           IDX(603)   // min size of point that is distance-shrunk (EXT_point_parameters)
#define RND_PRIM_POINT_THRESHOLD          IDX(604)   // size of point at which it starts alpha fading (EXT_point_parameters)
#define RND_PRIM_LINE_SMOOTH_ENABLE       IDX(605)   // enable/not line anti-aliasing
#define RND_PRIM_LINE_WIDTH               IDX(606)   // line width in pixels (float)
#define RND_PRIM_LINE_STIPPLE_ENABLE      IDX(607)   // enable/not line stippling
#define RND_PRIM_POLY_MODE                IDX(608)   // one of point, line, or fill
#define RND_PRIM_POLY_STIPPLE_ENABLE      IDX(609)   // enable/not polygon stippling
#define RND_PRIM_POLY_SMOOTH_ENABLE       IDX(610)   // do/don't alpha smooth polygon edges
#define RND_PRIM_POLY_OFFSET_ENABLE       IDX(611)   // enable/disable polygon Z offset
#define RND_PRIM_STIPPLE_BYTE             IDX(612)   // called for each byte of the poly and line stipple patterns on restart points.
#define RND_PRIM_PN_TRI_TESS_LEVEL        IDX(613)   // PN triangle sub-tesselation level (0 disables)
#define RND_PRIM_PN_TRI_POINT_MODE        IDX(614)   // PN triangl vertex interpolation mode
#define RND_PRIM_PN_TRI_NORMAL_MODE       IDX(615)   // PN triangle normal interpolation mode
#define RND_PRIM_NUM_PICKERS                   16

#define RND_TXU_ENABLE_MASK               IDX(700)   // bit mask of which textures are enabled
#define RND_TXU_WRAPMODE                  IDX(701)   // how to handle out-of-range S,T coordinates
#define RND_TXU_MIN_FILTER                IDX(702)   // how to map N texels down to N-M pixels
#define RND_TXU_MAG_FILTER                IDX(703)   // how to map N texels up to N+M pixels
#define RND_TXU_MIN_LAMBDA                IDX(704)   // min lambda (mipmap factor) (float)
#define RND_TXU_MAX_LAMBDA                IDX(705)   // max lambda (mipmap factor) (float)
#define RND_TXU_BIAS_LAMBDA               IDX(706)   // bias lambda (mipmap factor) (float)
#define RND_TXU_MAX_ANISOTROPY            IDX(707)   // max change in lambda across polygon
#define RND_TXU_BORDER_COLOR              IDX(708)   // border color r,g,b (float)
#define RND_TXU_BORDER_ALPHA              IDX(709)   // border alpha (float)
#define RND_TXSH_DOT_UNSIGNED_MAPPING     IDX(710)   // how to convert unsigned rgba into integers for dot-product texture shaders
#define RND_TXSH_PROG                     IDX(711)   // which texture shader program
                                                     //
                                                     // (4 txus) These require a normal-map followed by 3 dot-product ops in a row.
#define RND_TXSH_PROG_DotDotCube                0    // also requires a color cube texture
#define RND_TXSH_PROG_DotDot3d                  1    // also requires a color 3d texture
#define RND_TXSH_PROG_DotDotReflect             2    // also requires a color cube texture
#define RND_TXSH_PROG_DotDotEyeReflect          3    // also requires a color cube texture
#define RND_TXSH_PROG_DotDiffuseReflect         4    // also requires a color cube texture
#define RND_TXSH_PROG_DotDiffuseEyeReflect      5    // also requires a color cube texture
                                                     //
                                                     // (3 txus) These require a normal-map followed by 2 dot-product ops in a row.
#define RND_TXSH_PROG_Dot2d                     6    // also requires a 2d or rect texture
#define RND_TXSH_PROG_DotDepthReplace           7    // new frag depth = dotP/dotC
                                                     //
                                                     // (2 txus) Requires a previous unsigned color or hilo followed by just one dot op.
#define RND_TXSH_PROG_Dot1d                     8    // also requires a 1d texture
#define RND_TXSH_PROG_DotDepthReplaceAffine     9    // new frag depth = dotC  (only one depth-replace op allowed at a time)
#define RND_TXSH_PROG_DotPassThru              10    // new color = dotC
                                                     //
                                                     // (2 txus) These require a DSDT texture followed by a 2d or rect color texture.
#define RND_TXSH_PROG_Offset2d                 11    //
#define RND_TXSH_PROG_Offset2dProject          12    // (does the divide by q)
#define RND_TXSH_PROG_OffsetScale2d            13    // prev must be dsdt_mag or dsdt_mag_vib
#define RND_TXSH_PROG_OffsetScale2dProject     14    // prev must be dsdt_mag or dsdt_mag_vib (does the divide by q)
                                                     //
                                                     // (2 txus) These require a Hilo texture followed by a 2d or rect color texture.
#define RND_TXSH_PROG_HiloOffset2d             15    //
#define RND_TXSH_PROG_HiloOffset2dProject      16    // (does the divide by q)
                                                     //
                                                     // (2 txus)
#define RND_TXSH_PROG_DependentAR              17    // prev must be unsigned color, cur must be 2d or rect
#define RND_TXSH_PROG_DependentGB              18    // prev must be unsigned color, cur must be 2d or rect
#define RND_TXSH_PROG_DependentHILO            19    // prev must be HILO, cur must be 2d or rect
#define RND_TXSH_PROG_DependentRgb3d           20    // prev must be unsigned color, cur must be 3d
#define RND_TXSH_PROG_DependentRgbCube         21    // prev must be unsigned color, cur must be cube
                                                     //
                                                     // (1 txu) These are simple texture ops that depend only on themselves.
#define RND_TXSH_PROG_TxNd                     22    // traditional texture lookup
#define RND_TXSH_PROG_PassThru                 23    // rgba = strq
#define RND_TXSH_PROG_Cull                     24    // kill frag depending on strq <=> 0
#define RND_TXSH_PROG_None                     25    // disable txu
#define RND_TXSH_PROG_NUM_PROGS                26
#define RND_TXSH_CULL_MODES               IDX(712)   // cull mode array[4] for the texture shader fragment cull operation
#define RND_TX_DIM                        IDX(713)   // texture dimensionality: 2D, 3D, cube-map, etc.
#define RND_TX_WIDTH                      IDX(714)   // texture width in texels at base mipmap level
#define RND_TX_HEIGHT                     IDX(715)   // texture height in texels at base mipmap level
#define RND_TX_BASE_MM                    IDX(716)   // texture base mipmap level (probably 0, for no minification)
#define RND_TX_MAX_MM                     IDX(717)   // texture max mipmap level (probably 100, for go all the way down to 1x1)
#define RND_TX_NAME                       IDX(718)   // which pretty image should we load into the texture memory?
#define RND_TX_NAME_Checkers                    0    //   value for RND_TX_NAME
#define RND_TX_NAME_Ripple                      1    //   value for RND_TX_NAME
#define RND_TX_NAME_Random                      2    //   value for RND_TX_NAME
#define RND_TX_NAME_Spotlight                   3    //   value for RND_TX_NAME
#define RND_TX_NAME_Bricks                      4    //   value for RND_TX_NAME
#define RND_TX_NAME_SolidColor                  5    //   value for RND_TX_NAME
#define RND_TX_NAME_ShortMats                   6    //   value for RND_TX_NAME
#define RND_TX_NAME_LongMats                    7    //   value for RND_TX_NAME
#define RND_TX_NAME_WalkingOnes                 8    //   value for RND_TX_NAME
#define RND_TX_NAME_WalkingZeros                9    //   value for RND_TX_NAME
#define RND_TX_NAME_SuperZeroOne               10    //   value for RND_TX_NAME
#define RND_TX_NAME_ByteSuper                  11    //   value for RND_TX_NAME
#define RND_TX_NAME_CheckerBoard               12    //   value for RND_TX_NAME
#define RND_TX_NAME_InvCheckerBd               13    //   value for RND_TX_NAME
#define RND_TX_NAME_WalkSwizzle                14    //   value for RND_TX_NAME
#define RND_TX_NAME_WalkNormal                 15    //   value for RND_TX_NAME
#define RND_TX_NAME_DoubleZeroOne              16    //   value for RND_TX_NAME
#define RND_TX_NAME_TripleSuper                17    //   value for RND_TX_NAME
#define RND_TX_NAME_QuadZeroOne                18    //   value for RND_TX_NAME
#define RND_TX_NAME_TrippleWhammy              19    //   value for RND_TX_NAME
#define RND_TX_NAME_IsolatedOnes               20    //   value for RND_TX_NAME
#define RND_TX_NAME_IsolatedZeros              21    //   value for RND_TX_NAME
#define RND_TX_NAME_SlowFalling                22    //   value for RND_TX_NAME
#define RND_TX_NAME_SlowRising                 23    //   value for RND_TX_NAME
#define RND_TX_NAME_Address                    24    //   value for RND_TX_NAME
#define RND_TX_NAME_Tga                        25    //   value for RND_TX_NAME
#define RND_TX_INTERNAL_FORMAT            IDX(719)   // how the driver should store the texture in memory: RGB, A, RGBA, compression, bits per color, etc.
#define RND_TX_DEPTH                      IDX(720)   // depth in pixels of 3D textures
#define RND_TX_HAS_BORDER                 IDX(721)   // texture has 1 pixel border (add 2 to dimensions)
#define RND_TX_SHADER_ENABLE              IDX(722)   // enable texture shader extension
#define RND_TX_NUM_OBJS                   IDX(723)   // Number of texture objects to load at each restart point
#define RND_TX_MAX_TOTAL_BYTES            IDX(724)   // Max total bytes of texture memory to consume when loading textures at a restart point.
#define RND_TX_FRAG_PROG_ENABLE           IDX(725)   // Do/don't enable fragment programs.
#define RND_TX_FRAG_PROG_IS_COMBINER      IDX(726)   // Do/don't output 4 tx results for combiner to use
#define RND_TX_FRAG_PROG_NUM_OPS          IDX(727)   // how many opcodes in each fragment program (1 to 1024)
#define RND_TX_FRAG_PROG_OPCODE           IDX(728)   // which fragment program opcode to use next when generating programs
#define RND_TX_FRAG_PROG_OPCODE_add             0    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_cos             1    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_ddx             2    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_ddy             3    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_dp3             4    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_dp4             5    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_dst             6    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_ex2             7    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_flr             8    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_frc             9    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_kil            10    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_lg2            11    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_lit            12    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_lrp            13    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_mad            14    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_max            15    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_min            16    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_mov            17    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_mul            18    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_pow            19    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_rcp            20    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_rfl            21    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_rsq            22    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_seq            23    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_sfl            24    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_sge            25    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_sgt            26    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_sin            27    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_sle            28    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_slt            29    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_sne            30    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_str            31    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_sub            32    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_tex            33    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_txd            34    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_txp            35    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_x2d            36    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_pk2h           37    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_pk2us          38    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_pk4b           39    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_pk4ub          40    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_up2h           41    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_up2us          42    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_up4b           43    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_up4ub          44    //   value for RND_TX_FRAG_PROG_OPCODE
#define RND_TX_FRAG_PROG_OPCODE_NUM_OPCODES    45
#define RND_TX_FRAG_PROG_OP_PREC          IDX(729)   // float/half precision
#define RND_TX_FRAG_PROG_PREC_NONE              0    //
#define RND_TX_FRAG_PROG_PREC_X                 1    //
#define RND_TX_FRAG_PROG_PREC_H                 2    //
#define RND_TX_FRAG_PROG_PREC_R                 4    //
#define RND_TX_FRAG_PROG_OUT_REG          IDX(730)   // Which register to use for frag prog opcode result?
#define RND_TX_FRAG_PROG_REG_H                  0    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_R                 64    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_HC                96    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_RC                97    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_literal           98    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_oColH             99    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_oColR            100    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_oDpr             101    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_oTx0             102    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_oTx1             103    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_oTx2             104    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_oTx3             105    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_fWpos            106    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_fCol0            107    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_fCol1            108    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_fFogc            109    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_fTex0            110    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_fTex1            111    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_fTex2            112    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_fTex3            113    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_fTex4            114    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_fTex5            115    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_fTex6            116    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_fTex7            117    //   values for *REG pickers
#define RND_TX_FRAG_PROG_REG_NUM_REGS         118
#define RND_TX_FRAG_PROG_INDEX            IDX(731)   // Which fragment program to use
#define RND_TX_FRAG_PROG_OUT_CC           IDX(732)   // Output register condition code.
#define RND_TX_FRAG_PROG_OUT_CC_eq              0    //   value for RND_TX_FRAG_PROG_OUT_CC
#define RND_TX_FRAG_PROG_OUT_CC_ge              1    //   value for RND_TX_FRAG_PROG_OUT_CC
#define RND_TX_FRAG_PROG_OUT_CC_gt              2    //   value for RND_TX_FRAG_PROG_OUT_CC
#define RND_TX_FRAG_PROG_OUT_CC_le              3    //   value for RND_TX_FRAG_PROG_OUT_CC
#define RND_TX_FRAG_PROG_OUT_CC_lt              4    //   value for RND_TX_FRAG_PROG_OUT_CC
#define RND_TX_FRAG_PROG_OUT_CC_ne              5    //   value for RND_TX_FRAG_PROG_OUT_CC
#define RND_TX_FRAG_PROG_OUT_CC_tr              6    //   value for RND_TX_FRAG_PROG_OUT_CC
#define RND_TX_FRAG_PROG_OUT_CC_fl              7    //   value for RND_TX_FRAG_PROG_OUT_CC
#define RND_TX_FRAG_PROG_OUT_CC_none            8    //   value for RND_TX_FRAG_PROG_OUT_CC
#define RND_TX_FRAG_PROG_OUT_MASK         IDX(733)   // Output register mask
#define RND_TX_FRAG_PROG_OP_SETCC         IDX(734)   // Do/don't set condition code.
#define RND_TX_FRAG_PROG_SWIZZLE          IDX(735)   // input or CC swizzle
#define RND_TX_FRAG_PROG_SWIZZLE_xyzw           1    //   value for RND_TX_FRAG_PROG_SWIZZLE
#define RND_TX_FRAG_PROG_SWIZZLE_shuffle        2    //   value for RND_TX_FRAG_PROG_SWIZZLE
#define RND_TX_FRAG_PROG_SWIZZLE_random         3    //   value for RND_TX_FRAG_PROG_SWIZZLE
#define RND_TX_FRAG_PROG_IN_ABS           IDX(736)   // do/don't take absolute value of frag prog opcode input
#define RND_TX_FRAG_PROG_IN_NEG           IDX(737)   // do/don't negate frag prog opcode input
#define RND_TX_FRAG_PROG_OP_SATURATE      IDX(738)   // do/don't clamp result to [0, 1].
#define RND_TX_DRIVER_MMGEN               IDX(739)   // do/don't use SGIX_generate_mipmaps when loading texture
#define RND_TX_CONST_COLOR                IDX(740)   // Color for use by RND_TX_NAME_SolidColor.
#define RND_TX_NUM_PICKERS                     41

#define RND_VX_PRIMITIVE                  IDX(800)   // which primitive to draw: GL_POINTS, GL_TRIANGLES, etc.
#define RND_VX_PRIMITIVE_Solid                 10    //   (instead of GL_POINT .. GL_POLYGON, draws a "solid" closed convex mesh as tri-strips)
#define RND_VX_SEND_METHOD                IDX(801)   // how should vertexes be sent?
#define RND_VX_SEND_METHOD_Immediate            0    // use glBegin() glEnd(), with data passed on the stack.
#define RND_VX_SEND_METHOD_Vector               1    // use glBegin() glEnd(), with data passed as pointers.
#define RND_VX_SEND_METHOD_Array                2    // use glBegin() glEnd(), with vertex arrays and glArrayElement()
#define RND_VX_SEND_METHOD_DrawElements         3    // use glDrawElements() with vertex arrays and an index array
#define RND_VX_SEND_METHOD_DrawArrays           4    // use glDrawArray() with vertex arrays and sequential indexing.
#define RND_VX_SEND_METHOD_DrawElementArray     5    // use glDrawElementArrayNV() with vertex arrays and vertex index arrays.
#define RND_VX_PER_LOOP                   IDX(802)   // number of random vertexes to generate within each bbox
#define RND_VX_W                          IDX(803)   // vertex W (float)
#define RND_VX_EDGE_FLAG                  IDX(804)   // is this an edge or not (for wireframe mode)
#define RND_VX_MEMORY_TYPE                IDX(805)   // Agp/Pci/Fb memory for vertex arrays?
#define RND_VX_WEIGHT                     IDX(806)   // per-vertex weighting for nv10 skinning (float)
#define RND_VX_ARRAY_RANGE_ENABLE         IDX(807)   // enable NV_vertex_array_range extension
#define RND_VX_COMPILED_ENABLE            IDX(808)   // use compiled_vertex_arrays extension on this send
#define RND_VX_SECCOLOR_BYTE              IDX(809)   // secondary (specular) color r, g, or b
#define RND_VX_COLOR_BYTE                 IDX(810)   // primary color (material color) r,g,b
#define RND_VX_ALPHA_BYTE                 IDX(811)   // primary color (material color) transparency
#define RND_VX_NEW_GROUP                  IDX(812)   // called 17 times for laying out vertex array data: if always 0, all vertexes in one struct, max stride.
#define RND_VX_DO_SHUFFLE_STRUCT          IDX(813)   // if false, (if if not vxarray), don't shuffle the order of vx attribs in vertex struct.
#define RND_VX_PRIM_RESTART               IDX(814)   // do/don't use NV_primitive_restart with vertex arrays.
#define RND_VX_COLOR_WHITE                IDX(815)   // do/don't force color==white
#define RND_VX_NUM_PICKERS                     16

#define RND_VXFMT_COORD_SIZE              IDX(900)   // 2 to 4 for xy, xyz, or xyzw coordinates
#define RND_VXFMT_COORD_DATA_TYPE         IDX(901)   // GL_FLOAT or GL_SHORT.
#define RND_VXFMT_HAS_NORMAL              IDX(902)   // has/not got a normal vector per vertex
#define RND_VXFMT_NORMAL_DATA_TYPE        IDX(903)   // double, float, short, etc.
#define RND_VXFMT_HAS_PRICOLOR            IDX(904)   // has/not got a color per vertex
#define RND_VXFMT_HAS_EDGE_FLAG           IDX(905)   // has/not got edge flag per vertex
#define RND_VXFMT_TXU_COORD_SIZE          IDX(906)   // number of texture coordinates: 2,3 == st, str
#define RND_VXFMT_HAS_SECCOLOR            IDX(907)   // do/don't send per-vertex attributes that don't have their own pickers: WGHT,COL1,FOGC,6,7
#define RND_VXFMT_HAS_FOG_COORD           IDX(908)   // do/don't send per-vertex fog coords
#define RND_VXFMT_HAS_WEIGHT              IDX(909)   // do/don't send per-vertex nv10/15/11 skinning weight
#define RND_VXFMT_INDEX_TYPE              IDX(910)   // data type for vertex indexes
#define RND_VXFMT_NUM_PICKERS                  11

#define RND_TSTCTRL_STOP                 IDX(1000)   // (picked before all other pickers in a loop) stop testing, return current pass/fail code
#define RND_TSTCTRL_SUPERVERBOSE         IDX(1001)   // (picked after all other pickers in a loop) do/don't print picks just before sending them
#define RND_TSTCTRL_SKIP                 IDX(1002)   // (picked after SUPERVERBOSE) do/don't render (if returns nonzero/true, nothing is sent to the GL driver this loop)
#define RND_TSTCTRL_FLUSH                IDX(1003)   // (picked after the send on each loop) do/don't call glFinish() after the draw.
#define RND_TSTCTRL_LOGMODE              IDX(1004)   // (picked just after STOP) self-check logging/checking of generated textures and random picks
#define RND_TSTCTRL_LOGMODE_Skip                0    //   value for RND_TSTCTRL_LOGMODE
#define RND_TSTCTRL_LOGMODE_Store               1    //   value for RND_TSTCTRL_LOGMODE
#define RND_TSTCTRL_LOGMODE_Check               2    //   value for RND_TSTCTRL_LOGMODE
#define RND_TSTCTRL_EMIT_FAKE_GL         IDX(1005)   // (picked after skip) do/don't emit fakegl source code
#define RND_TSTCTRL_NUM_PICKERS                 6

